@@ -3,10 +3,10 @@
 # latexdiff-vc  - wrapper script for applying latexdiff to rcs managed files
 #                 and for automatised creation of postscript or pdf from difference file
 #
-#   Copyright (C) 2005-13  F J Tilmann (tilmann@gfz-potsdam.de)
+#   Copyright (C) 2005-16  F J Tilmann (tilmann@gfz-potsdam.de)
 #
 # Repository:         https://github.com/ftilmann/latexdiff
-# CTAN page:          http://www.ctan.org/tex-archive/support/latexdiff
+# CTAN page:          http://www.ctan.org/pkg/latexdiff
 #
 #
 #   Contributors: S Utcke, H Bruyninckx, C Junghans
@@ -25,6 +25,24 @@
 #
 # Detailed usage information at the end of the file
 #
+# TODO/IDEAS: - option to call external pre-processing codes
+# version 1.3.0 ((7 October 2018)
+#    - option --only-changes with hyperref will suppress hyperrefs (pull request jprotze)_
+#    - option --only-changes now moves (rather than copies) file with only changes 
+# version 1.2.1 (22 June 2017):
+#    - update: use qpdf (instead of pdftk) to select particular pages of output (pull request #102 submitted by Tom Scogland via github). This was necessary because pdftk is being deprecated. pdftk is still used as a fall-back
+#    - bug fix: --hg option was not recognised (partially fixes github issue #93 )
+#    - wrap passed-through options to latexdiff in quotation marks (fix github issue #58 )
+#    - program names for latexdiff, latex, dvips, bibtex configurable (fixes issue #40)
+#
+# version 1.2.0:
+#    - deprecation fix: left brace in RegEx now needs to be escaped
+#
+# version 1.1.1:
+#    - better detection of RCS system 
+#    - undocumented option --debug/--nodebug to override default setting for debug mode (Default: 0)#
+#    - bug fix: --flatten option combined with --pdf caused confusion of old and new file
+#
 # version 1.1.0:
 #
 #      - with option --flatten and version control option, checkout the whole tree into a temporary directory
@@ -53,22 +71,40 @@ use strict ;
 use warnings ;
 
 my $versionstring=<<EOF ;
-This is LATEXDIFF-VC 1.1.0
-  (c) 2005-2015 F J Tilmann
+This is LATEXDIFF-VC 1.3.1
+  (c) 2005-2020 F J Tilmann
 EOF
 
+# output debug and intermediate files, set to 0 in final distribution
+my $debug=0; 
+
 
 # Option names
-my ($version,$help,$fast,$so,$postscript,$pdf,$onlychanges,$flatten,$force,$dir,$cvs,$rcs,$svn,$git,$hg,$diffcmd,$patchcmd,@revs);
+my ($version,$help,$fast,$showconfig,$so,$postscript,$pdf,$onlychanges,$flatten,$force,$dir,$cvs,$rcs,$svn,$git,$hg,
+    $run,$rundvi2,$diffcmd,$patchcmd,@revs,@configlist);
+# for passing through config options to latexdiff:
+my ($configlatexdiff,@config,$config,$assign);
+
 # Preset Variables
-my $latexdiff="latexdiff"; # Program for making the comparison
-my $latexcmd="latex"; # latex compiler if not further identified
+#my $latexdiff="latexdiff"; # Program for making the comparison
+#my $latexcmd="latex"; # latex compiler if not further identified
 my $vc="";
 my $tempdir=tempdir(CLEANUP => 1);   # generate a temp dir, which will automatically be deleted at program exit
 # Variables
 my ($file1,$file2,$diff,$diffbase,$rootdir,$answer,$options,$infile,$append,$dirname,$cwd);
 my (@files,@ldoptions,@tmpfiles,@ptmpfiles,@difffiles,$extracomp); # ,
 
+
+# keep commands to be used in config hash
+my (%CFG)=( 
+   LATEX     => "latex",
+   LATEXDIFF => "latexdiff",
+   DVI2      => "dvips",
+   BIBTEX    => "bibtex"
+   );
+
+
+
 Getopt::Long::Configure('pass_through','bundling');
 
 GetOptions('revision|r:s' => \@revs,
@@ -78,6 +114,8 @@ GetOptions('revision|r:s' => \@revs,
            'git' => \$git,
            'hg' => \$hg,
            'dir|d:s' => \$dir,
+	   'run' => \$run,
+	   'dvi' => \$rundvi2,
 	   'fast' => \$fast,
 	   'so' => \$so,
            'postscript|ps' => \$postscript,
@@ -85,8 +123,14 @@ GetOptions('revision|r:s' => \@revs,
            'force' => \$force,
 	   'only-changes' => \$onlychanges,
 	   'flatten:s' => \$flatten,
+	   'config|c=s' => \@configlist,
+	   'show-config' => \$showconfig,
            'version' => \$version,
-	   'help|h' => \$help);
+	   'help|h' => \$help,
+	   'debug!' => \$debug);
+
+##print STDERR "DEBUG 1:revs($#revs): " . join(":",@revs) . "\n";
+##print STDERR "DEBUG 1:ARGV($#ARGV): " . join(":",@ARGV) . "\n";
 
 $extracomp = join(" ",grep(/BAR/,@ARGV)); # special latexdiff options requiring additional compilation
 
@@ -99,11 +143,11 @@ if ( $version ) {
 }
 
 if ( $so ) {
-  $latexdiff='latexdiff-so';
+  $CFG{LATEXDIFF}='latexdiff-so';
 }
 if ( $fast ) { 
   die "Cannot specify more than one of --fast or --so " if ($so);
-  $latexdiff='latexdiff-fast';
+  $CFG{LATEXDIFF}='latexdiff-fast';
 }
 
 if ( $cvs ) {
@@ -121,26 +165,85 @@ if ( $git ) {
   die "Cannot specify more than one of --cvs, --rcs, --svn --git or --hg." if ($vc);
   $vc="GIT";
 }
+if ( $hg ) {
+  die "Cannot specify more than one of --cvs, --rcs, --svn --git or --hg." if ($vc);
+  $vc="HG";
+}
+
+if ( $pdf ) {
+  $CFG{LATEX} = "pdflatex";
+  $run=1;
+} elsif ( $postscript ) {
+  $CFG{LATEX} = "latex";
+  $rundvi2=1;
+}
+
+if ( $rundvi2 ) {
+  $run=1;
+}
+
+# setting config variables from command line. Explicitly set configuration options take precedence over convenience options,
+# therefore this is executed last
+@config=();
+foreach $config ( @configlist ) {
+  if (-f $config || lc $config eq '/dev/null' ) {
+    open(FILE,$config) or die ("Couldn't open configuration file $config: $!");
+    while (<FILE>) {
+      chomp;
+      next if /^\s*#/ || /^\s*%/ || /^\s*$/ ;
+      push (@config,$_);
+    }
+    close(FILE);
+  }
+  else {
+#    foreach ( split(",",$config) ) {
+#      push @config,$_;
+#    }
+     push @config,split(",",$config)
+  }
+}
+
+$configlatexdiff="";
+foreach $assign ( @config ) {
+  $assign=~ m/\s*(\w*)\s*=\s*(\S*)\s*$/ or die "Illegal assignment $assign in configuration list (must be variable=value)";  
+  if ( defined($CFG{$1})) {
+    # known latexdiff-vc option
+    $CFG{$1}=$2;
+  } else {
+    # unknown option, assume it is a latexdiff configuration variable
+    $configlatexdiff .= "$1=$2,";
+  }
+}
+# delete last comma inv $configlatexdiff if present
+$configlatexdiff =~ s/,$// ;
+
+if ($showconfig) {
+  print "# Configuration variables (latexdiff-vc):\n";
+  foreach ( keys %CFG ) {
+    print "$_=$CFG{$_}\n";
+  }
+  exit 0;
+}
 
 
 # check whether the first file name or first passed-through option for latexdiff got misinterpreted as an option to an empty -r option
 if ( @revs && ( -f $revs[$#revs] || $revs[$#revs] =~ /^-/ ) ) {
-  unshift @ARGV,$revs[$#revs];
+  push @ARGV,$revs[$#revs];
   $revs[$#revs]="";
 }
 # check whether the first file name or first passed-through option for latexdiff got misinterpreted as an option to an empty -d option
 if ( defined($dir) && ( -f $dir || $dir =~ /^-/ ) ) {
-  unshift @ARGV,$dir;
+  push @ARGV,$dir;
   $dir="";
 }
 # check whether the first file name or first passed-through option for latexdiff got misinterpreted as an option to an empty --flatten option
+
 if ( defined($flatten) && ( -f $flatten || $flatten =~ /^-/ ) ) {
   unshift @ARGV,$flatten;
   $flatten="";
 }
 
-
-#print "DEBUG: latexdiff-vc command line: ", join(" ",@ARGV), "\n"; 
+print STDERR "DEBUG: PDF $pdf latexdiff-vc command line: ", join(" ",@ARGV), "\n" if $debug; 
 
 $file2=pop @ARGV;
 ( defined($file2) && $file2 =~ /\.(tex|bbl|flt)$/ ) or pod2usage("Must specify at least one tex, bbl or flt file");
@@ -158,7 +261,7 @@ if (! $vc && scalar(@revs)>0 ) {
     $vc="GIT";
   } elsif ( $0 =~ /-hg$/ ) {
     $vc="HG";
-  } elsif ( -e "$file2,v" ) {
+  } elsif ( -e "$file2,v" || -d "RCS" ) {
     print STDERR "Guess you are using RCS ...\n";
     $vc="RCS";
   } elsif ( -d ".svn" ) {
@@ -192,7 +295,7 @@ if ( scalar(@revs)>0 ) {
     $diffcmd  = "rcsdiff -u -r";  
     $patchcmd = "patch -R -p0";
   } elsif ( $vc eq "SVN" ) {
-    $diffcmd  = "svn diff -r ";  
+    $diffcmd  = "svn diff --internal-diff -r ";  
     $patchcmd = "patch -R -p0";
   } elsif ( $vc eq "GIT" ) {
     $diffcmd  = "git diff -r --relative --no-prefix ";  
@@ -201,7 +304,7 @@ if ( scalar(@revs)>0 ) {
     # $diffcmd  = "git diff ";
     # $patchcmd = "patch -R -p1";
   } elsif ( $vc eq "HG" ) {
-    $diffcmd  = "hg diff -r";
+    $diffcmd  = "hg diff --root . -r";
     $patchcmd = "patch -R -p1";
   } else {
     print STDERR "Unknown versioning system $vc \n";
@@ -225,26 +328,34 @@ while( $file1=pop @ARGV ) {
   }
 }
 
+if ( length $configlatexdiff >0 ) {
+  push @ldoptions, "--config=$configlatexdiff";
+}
+
 if ( defined($flatten) ) {
   push @ldoptions, "--flatten" ;
 }
 
+if ( defined($debug) && $debug ) {
+  push @ldoptions, "--debug" ;
+}
+
 # impose ZLABEL subtype if --only-changes option
 if ( $onlychanges ) {
-  push @ldoptions, "-s", "ZLABEL" ;
+  push @ldoptions, "-s", "ZLABEL","-f","IDENTICAL","--no-links" ;
 }
 
 if ( scalar(@revs) == 0 ) {
   pod2usage("When -r option is not used, two .tex files (old and new) must be given on the command line") unless @files==2;
-  warn "Option -flatten should normally be combined with -r option (Results will probably not be as expected)" if $flatten;
+  warn "Option -flatten should normally be combined with -r option (Results will probably not be as expected)" if defined($flatten);
     #  compare two files
   $file1=shift @files ;
 } else {
   # revision control
-  if ( $flatten ) {
+  if ( defined($flatten) ) {
     pod2usage("Only one root file must be given  if --flatten option is used with version control") if  @files != 1;
     $tempdir='.' if ( $flatten eq "keep-intermediate" );
-    print "flatten tempdir |$tempdir|";
+    print STDERR "flatten tempdir |$tempdir|\n" if $debug;
     if ( $vc eq "SVN" ) {
       my (@infoout)=`svn info`;
       my (@urlline) = grep(/^URL:/, @infoout);
@@ -277,7 +388,12 @@ if ( ($vc eq "SVN" || $vc eq "CVS") && scalar(@revs)) {
   length($revs[0]) > 0 or $revs[0]="HEAD";
 }
 
-#exit ; 
+if ($vc eq "HG" && scalar(@revs)) {
+  length($revs[$#revs]) > 0 or $revs[$#revs]="tip";
+  length($revs[0]) > 0 or $revs[0]="tip";
+}
+
+print STDERR "DEBUG LDOptions($#ldoptions): " . join(":",@ldoptions) . "\n" if $debug;
 
 
 # cycle through all files
@@ -342,7 +458,11 @@ while ( $infile=$file2=shift @files ) {
   system("mkdir -p $dirname") unless ( -e $dirname );
 
   # Remaining options are passed to latexdiff
-  $options = join(" ",@ldoptions);
+  if (scalar(@ldoptions) > 0 ) {
+    $options = "\'" . join("\' \'",@ldoptions) . "\'";
+  } else {
+    $options = "";
+  }
    
   if ( -e $diff && ! $force ) {
     print STDERR "OK to overwrite existing file $diff (y/n)? ";
@@ -352,13 +472,13 @@ while ( $infile=$file2=shift @files ) {
       die "Abort ... " ;
     }
   }
-  print "Running $latexdiff\n";
-  unless ( system("$latexdiff $options \"$file1\" \"$file2\" > \"$diff\"") == 0 ) { 
-    print STDERR  "Something went wrong in $latexdiff. Deleting $diff and abort\n" ; unlink $diff ; exit(5) 
+  print STDERR "Running: $CFG{LATEXDIFF}  $options \"$file1\" \"$file2\" > \"$diff\"\n";
+  unless ( system("$CFG{LATEXDIFF} $options \"$file1\" \"$file2\" > \"$diff\"") == 0 ) { 
+    print STDERR  "Something went wrong in $CFG{LATEXDIFF}. Deleting $diff and abort\n" ; unlink $diff ; exit(5) 
   };
   print "Generated difference file $diff\n";
    
-  if ( ( $postscript or $pdf ) and !( scalar(@revs) && greptex( qr/\\document(?:class|style)/ , $diff ) ) ) {
+  if ( $run and !( scalar(@revs) && greptex( qr/\\document(?:class|style)/ , $diff ) ) ) {
     # save filename for later processing if postscript or pdf conversion is requested and either two-file mode was used (scalar(@revs)==0) or the diff file contains documentclass statement (ie. is a root document)
     push @difffiles, $diff ;
   }
@@ -378,50 +498,64 @@ foreach $diff ( @difffiles ) {
   # adapt magically changebar styles to [pdftex] display driver if pdf output was selected
   if ( $pdf ) {
     system("sed \"s/Package\\[dvips\\]/Package[pdftex]/\" \"$diff\" > \"$diff.tmp$$\" ; \\mv \"$diff.tmp$$\" \"$diff\"");
-    $latexcmd = "pdflatex";
-  } elsif ( $postscript ) {
-    $latexcmd = "latex";
   }
 
-  if ( $pdf | $postscript ) {
-  print STDERR "PDF: $pdf Postscript: $postscript cwd $cwd\n";
+  if ( $run ) {
+    print STDERR "PDF: $pdf Postscript: $postscript cwd $cwd\n" if $debug;
 
-  if ( system("grep -q \'^[^%]*\\\\bibliography\' \"$diff\"") == 0 ) { 
-    system("$latexcmd --interaction=batchmode \"$diff\"; bibtex \"$diffbase\";");
-    push @ptmpfiles, "$diffbase.bbl","$diffbase.bbl" ; 
-  }
-
-  # if special needs, as CHANGEBAR
-  if ( $extracomp ) {
-    # print "Extracomp\n";
-    system("$latexcmd --interaction=batchmode \"$diff\";");
-  }
-
-  # final compilation
-  system("$latexcmd --interaction=batchmode \"$diff\";"); # needed if cross-refs
-  system("$latexcmd \"$diff\";"); # final, with possible error messages
-
-  if ( $postscript ) {
-    my $dvi="$diffbase.dvi";
-    my $ps="$diffbase.ps";
-    my $ppoption="";
-
-    if ( $onlychanges ) {
-      $ppoption="-pp ".join(",",findchangedpages("$diffbase.aux"));
+    if ( system("grep -q \'^[^%]*\\\\bibliography{\' \"$diff\"") == 0 ) { 
+      system("$CFG{LATEX} --interaction=batchmode \"$diff\"; $CFG{BIBTEX} \"$diffbase\";");
+      push @ptmpfiles, "$diffbase.bbl","$diffbase.bbl" ; 
     }
-    system("dvips $ppoption -o $ps $dvi");
-    push @ptmpfiles, "$diffbase.aux","$diffbase.log",$dvi ;
-    print "Generated postscript file $ps\n";
-  } 
-  elsif ( $pdf ) {
-    if ( $onlychanges ) {
-      my @pages=findchangedpages("$diffbase.aux");
-      system ("pdftk \"$diffbase.pdf\" cat " . join(" ",@pages) . " output \"$diffbase-changedpage.pdf\"")==0 or 
-               die ("Could not execute <pdftk $diffbase.pdf cat " . join(" ",@pages) . " output $diffbase-changedpage.pdf> . Return code: $?");
-      move("$diffbase-changedpage.pdf","$diffbase.pdf");
+
+    # if special needs, as CHANGEBAR
+    if ( $extracomp ) {
+      # print "Extracomp\n";
+      system("$CFG{LATEX} --interaction=batchmode \"$diff\";");
+    }
+
+    # final compilation
+    system("$CFG{LATEX} --interaction=batchmode \"$diff\";"); # needed if cross-refs
+    system("$CFG{LATEX} \"$diff\";"); # final, with possible error messages
+
+    if ( $rundvi2 ) {
+      my $dvi="$diffbase.dvi";
+      #my $ps="$diffbase.ps";
+      my $ppoption="";
+      
+      if ( $onlychanges ) {
+	$ppoption="-pp ".join(",",findchangedpages("$diffbase.aux"));
+      }
+      #system("$CFG{DVI2} $ppoption -o $ps $dvi");
+      system("$CFG{DVI2} $ppoption $dvi");
+      push @ptmpfiles, "$diffbase.aux","$diffbase.log",$dvi ;
+      #print STDERR "Generated postscript file $ps\n";
+    } elsif ( $run ) {
+      if ( $onlychanges ) {
+	my @pages=findchangedpages("$diffbase.aux");
+        my $gs = `which gs`;
+        $gs =~ s/^\s+|\s+$//g;
+        my $qpdf = `which qpdf`;
+        $qpdf =~ s/^\s+|\s+$//g;
+        my $pdftk = `which pdftk`;
+        $pdftk =~ s/^\s+|\s+$//g;
+        my $command;
+        if (-x $gs && `gs --version` >= 9.20) {
+          $command="gs -sDEVICE=pdfwrite -dNOPAUSE -dBATCH -dSAFER -sPageList=" . join(",", @pages) . " -sOutputFile=\"$diffbase-changedpage.pdf\" \"$diffbase.pdf\"";
+        } elsif (-x $pdftk) {
+          $command="pdftk \"$diffbase.pdf\" cat " . join(" ",@pages) . " output \"$diffbase-changedpage.pdf\"";
+        } elsif (-x $qpdf) {
+          $command="qpdf --linearize \"$diffbase.pdf\" --pages \"$diffbase.pdf\" " . join(",", @pages) . " -- \"$diffbase-changedpage.pdf\" ";
+        } else {
+          die ("could not find any of gs, pdftk or qpdf");
+        }
+        print "Executing ".$command;
+        system($command) == 0
+          or die("could not execute <".$command."> to strip pages. Return code: $?");
+	move("$diffbase-changedpage.pdf","$diffbase.pdf");
+      }
+      push @ptmpfiles, "$diffbase.aux","$diffbase.log";
     }
-    push @ptmpfiles, "$diffbase.aux","$diffbase.log";
-  }
   }
   unlink @ptmpfiles;
   chdir $cwd;
@@ -432,14 +566,14 @@ foreach $diff ( @difffiles ) {
 sub findchangedpages {
   my ($auxfile) =@_;
   my $j;
-  my (%pages);		# note that I use a hash with page numbers as keys and arbitrary values - this way I can use perl's built-in functions to get red of duplicates
+  my (%pages);		# note that I use a hash with page numbers as keys and arbitrary values - this way I can use Perl's built-in functions to get rid of duplicates
   my %start;
   open(AUX,$auxfile) or die ("Could open aux file $auxfile . System error: $!");
   while (<AUX>) {
-    if (m/\\zref\@newlabel{DIFchgb(\d*)}{.*\\abspage{(\d*)}}/ ) { 
+    if (m/\\zref\@newlabel\{DIFchgb(\d*)\}\{.*\\abspage\{(\d*)\}\}/ ) { 
       $start{$1}=$2; $pages{$2}=1;
     }
-    if (m/\\zref\@newlabel{DIFchge(\d*)}{.*\\abspage{(\d*)}}/) { 
+    if (m/\\zref\@newlabel\{DIFchge(\d*)\}\{.*\\abspage\{(\d*)\}\}/) { 
       if (defined($start{$1})) {
 	for ($j=$start{$1}; $j<=$2; $j++) {
 	  $pages{$j}=1;
@@ -464,6 +598,7 @@ sub checkout_dir {
   if ( $vc eq "SVN" ) {
     system("svn checkout -r $rev $rootdir $dirname")==0 or die "Something went wrong in executing:  svn checkout -r $rev $rootdir $dirname";
   } elsif ( $vc eq "GIT" ) {
+    $rev="HEAD" if length($rev)==0;
     system("git archive --format=tar $rev | ( cd $dirname ; tar -xf -)")==0 or die "Something went wrong in executing:  git archive --format=tar $rev | ( cd $dirname ; tar -xf -)";
   } elsif ( $vc eq "HG" ) {
     system("hg archive --type files -r $rev $dirname")==0 or die "Something went wrong in executing:  hg archive --type files -r $rev $dirname";
@@ -509,7 +644,7 @@ B<latexdiff-vc> [ F<latexdiff-options> ]  [ F<latexdiff-vc-options> ][ B<--posts
 =head1 DESCRIPTION
 
 I<latexdiff-vc> is a wrapper script that applies I<latexdiff> to a
-file, or multiple files under version control (CVS, RCS or SVN), and optionally runs the
+file, or multiple files under version control (git, subversion (SVN), mercurial (hg), CVS, RCS), and optionally runs the
 sequence of C<latex> and C<dvips> or C<pdflatex> commands necessary to
 produce pdf or postscript output of the difference tex file(s). It can
 also be applied to a pair of files to automatise the generation of difference
@@ -521,12 +656,14 @@ file in postscript or pdf format.
 
 =item B<--rcs>, B<--svn>, B<--cvs>, B<--git> or B<--hg>
 
-Set the version system. 
+Set the version control system used. 
 If no version system is specified, latexdiff-vc will venture a guess.
 
-latexdiff-cvs and latexdiff-rcs are variants of latexdiff-vc which default to 
+latexdiff-cvs, latexdiff-rcs etc are variants of latexdiff-vc which default to 
 the respective versioning system. However, this default can still be overridden using the options above.
 
+Note that hg needs to support the C<--root> option (version >= 2.9)
+
 =item B<-r>, B<-r> F<rev> or B<--revision>, B<--revision=>F<rev>
 
 Choose revision (under RCS, CVS, SVN, GIT or HG). One or two B<-r> options can be
@@ -536,7 +673,7 @@ specified, and they result in different behaviour:
 
 =item B<latexdiff-vc> -r F<file.tex> ...
 
-compares F<file.tex> with the most recent version checked into RCS.
+compares F<file.tex> with the most recently checked-in version checked.
 
 =item B<latexdiff-vc> -r F<rev1> F<file.tex> ...
 
@@ -547,7 +684,7 @@ compares F<file.tex> with revision F<rev1>.
 compares revisions F<rev1> and F<rev2> of F<file.tex>.
 
 Multiple files can be specified for all of the above options. All files must have the
-extension C<.tex>, though.
+extensions C<.tex>, C<.bbl>, or C<.flt>, though.
 
 =item B<latexdiff-vc>  F<old.tex> F<new.tex>
 
@@ -563,7 +700,7 @@ reported to stdout.
 Rather than appending the string C<diff> and optionally the version
 numbers given to the output-file, this will prepend a directory name C<diff> 
 to the
-original filename, creating the directory and subdirectories should they not exist already.  This is particularly useful in order to clone a
+original filename, creating the directory and sub-directories should they not exist already.  This is particularly useful in order to clone a
 complete directory hierarchy.  Optionally, a pathname F<path> can be specified, which is prepended instead of C<diff>.
 
 =item B<--flatten,--flatten=keep-intermediate>
@@ -575,6 +712,49 @@ The generic usage of this function is : C<latexdiff-vc --flatten -r rev1 [-r rev
 
 With C<--flatten=keep-intermediate>, the intermediate revision snapshots are kept in the current directory (Default is to store them in a temporary directory and delete them after generating the diff file.)
 
+
+=item B<--only-changes>
+
+Post-process the output such that only pages with changes on them are displayed. This requires the use of subtype ZLABEL 
+in latexdiff, which will be set automatically, but any manually set -s option will be overruled (also requires zref package to 
+be installed). (note that this option must be combined with --ps or --pdf to make sense)
+
+=item B<--force>
+
+Overwrite existing diff files without asking for confirmation. Default 
+behaviour is to ask for confirmation before overwriting an existing difference
+file.
+
+=item B<--run>
+
+run latex command on diff file after generation of diff file.
+
+=item B<--dvi>
+
+run latex and dvixxx commands after generation of diff file.
+
+=item B<-c configfile>
+=item B<--config var1=val1,var2=val2,...> or B<-c var1=val1,..>
+
+Set configuration variables for latexdiff and latexdiff-vc.  The option can be repeated to set different
+variables (as an alternative to the comma-separated list).
+Available variables for latexdiff-vc:
+
+=over 8
+
+=item C<LATEXDIFF> latexdiff command (e.g. latexdiff-fast, latexdiff-so). This command should support the option C<--interaction=batchmode>
+
+=item C<LATEX> latex command (e.g. pdflatex, lualatex)
+
+=item C<DVI2>  Command for conversion of dvi file (e.g. dvips, dvipdf)
+
+=item C<BIBTEX> Command replacing bibtex
+
+=back
+
+All other config variables are passed to latexdiff. Explicitly set configuration changes always override implicit
+changes by the following shortcut options B<--fast>, B<--so>, B<--ps> and B<--pdf>. 
+
 =item B<--fast> or B<--so>
 
 Use C<latexdiff-fast> or C<latexdiff-so>, respectively (instead of C<latexdiff>).
@@ -593,21 +773,15 @@ bibtex; latex; latex; dvips>.
 Generate pdf output from difference file using C<pdflatex>. This will
 run the sequence C<pdflatex; pdflatex> on the difference file, or
 C<pdflatex; bibtex; pdflatex; pdflatex> for files requiring bibtex.
+Note that this is not just a shortcut for setting configuration variable but also triggers 
+some special behaviour.
 
-=item B<--only-changes>
 
-Post-process the output such that only pages with changes on them are displayed. This requires the use of subtype ZLABEL 
-in latexdiff, which will be set automatically, but any manually set -s option will be overruled (also requires zref package to 
-be installsed). (note that this option must be combined with --ps or --pdf to make sense)
+=item B<--show-config>
 
-=item B<--force>
+Show values of configuration variables.
 
-Overwrite existing diff files without asking for confirmation. Default 
-behaviour is to ask for confirmation before overwriting an existing difference
-file.
-
-=item B<--help> or
-B<-h>
+=item B<--help> or B<-h>
 
 Show help text
 
@@ -625,20 +799,21 @@ L<latexdiff>
 
 =head1 PORTABILITY
 
-I<latexdiff-vc> uses external commands and is therefore
-limited to Unix-like systems. It also requires the a version control
+I<latexdiff-vc> uses external commands and is therefore dependent on the system architecture; it has been
+tested mainly on Unix-like systems. It also requires a version control
 system and latex to be installed on the system to make use of all features.  Modules from Perl 5.8
 or higher are required.
 
 =head1 BUG REPORTING
 
 Please submit bug reports using the issue tracker of the github repository page I<https://github.com/ftilmann/latexdiff.git>, 
-or send them to I<tilmann -- AT -- gfz-potsdam.de>.  Include the serial number of I<latexdiff-vc>
+or send them to I<tilmann -- AT -- gfz-potsdam.de>.  Include the version number of I<latexdiff-vc>
 (option C<--version>).
 
 =head1 AUTHOR
 
-Copyright (C) 2005-2015 Frederik Tilmann
+Version 1.2.1
+Copyright (C) 2005-2017 Frederik Tilmann
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License Version 3
