@@ -1,6 +1,6 @@
 #!/usr/bin/env wish
 
-# Copyright 2017-2019 Siep Kroonenberg
+# Copyright 2017-2020 Siep Kroonenberg
 
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
@@ -14,10 +14,6 @@ catch {rename send {}}
 # on windows, a wrapper takes care of this.
 if {$::tcl_platform(platform) ne "windows"} {
   set texbin [file dirname [file normalize [info script]]]
-  set savedir [pwd]
-  cd $texbin
-  set texbin [pwd]
-  cd $savedir
   # prepend texbin to PATH, unless it is already the _first_
   # path component
   set dirs [split $::env(PATH) ":"]
@@ -25,13 +21,84 @@ if {$::tcl_platform(platform) ne "windows"} {
     set ::env(PATH) "${texbin}:$::env(PATH)"
   }
   unset texbin
-  unset savedir
   unset dirs
 }
 
 set ::instroot [exec kpsewhich -var-value=TEXMFROOT]
 
-# declarations and utilities shared with install-tl-gui.tcl
+# try to read a configuration variable (gui-lang, tkfontscale)
+# from tlmgr config ($TEXMF[SYS]CONFIG/tlmgr/config
+# failure results in an empty string.
+# this proc will be invoked by tltcl if tlshell is the invoker of tltcl
+proc get_config_var {k} {
+  set v ""
+  set r [join [list {^\s*} $k {\s*=\s*(\S.*)$}] ""]
+  foreach tmf {"TEXMFCONFIG" "TEXMFSYSCONFIG"} {
+    if [catch {exec kpsewhich -var-value $tmf} d] {
+        break; # apparently there is not yet a TL installation
+    }
+
+    if [catch {open [file join $d "tlmgr" "config"] r} fid] continue
+    while 1 {
+      if [chan eof $fid] {
+        break
+      }
+      if [catch {chan gets $fid} l] break
+      # we can assume that $k contains no special characters
+      if [regexp $r $l m v] {
+        set v [regsub {\s*$} $v ""]
+        break
+      } else {
+        # regexp will have unset v
+        set v ""
+      }
+    }
+    chan close $fid
+    if {$v ne ""} break
+  }
+  return $v
+}
+
+proc save_config_var {k v} {
+  if [catch {exec kpsewhich -var-value "TEXMFCONFIG"} d] {return 0}
+  set d [file join $d "tlmgr"]
+  if [catch {file mkdir $d}] {return 0} ; # mkdir on existing dir is ok
+  set fn [file join $d "config"]
+  set oldlines [list]
+  set r [join [list {^\s*} $k {\s*=\s*(\S.*)$}] ""]
+  # read current config file
+  if [file exists $fn] {
+    if [catch {open $fn r} fid] {return 0}
+    set cnt 0
+    while 1 {
+      if [catch {chan gets $fid} l] break
+      if [chan eof $fid] break
+      incr cnt
+      if {! [regexp $r $l]} {
+          lappend oldlines $l
+      }
+      if {$cnt>40} break
+    }
+    catch {chan close $fid}
+  }
+  lappend oldlines "$k = $v"
+  if [catch {open $fn w} fid] {return 0}
+  foreach l $oldlines {
+    if [catch {puts $fid $l}] {
+      catch {chan close $fid}
+      return 0
+    }
+  }
+  catch {chan close $fid}
+  return 1
+}
+
+# tltcl: declarations and utilities shared with install-tl-gui.tcl
+# tltcl likes to know who was the invoker
+set ::invoker [file tail [info script]]
+if [string match -nocase ".tcl" [string range $::invoker end-3 end]] {
+  set ::invoker [string range $::invoker 0 end-4]
+}
 source [file join $::instroot "tlpkg" "tltcl" "tltcl.tcl"]
 
 # now is a good time to ask tlmgr for the _TL_ name of our platform
@@ -59,9 +126,6 @@ set ddebug 0
 # menus: disable tearoff feature
 option add *Menu.tearOff 0
 
-# for busy/idle indicators
-set ::busy [__ "Idle"]
-
 proc search_nocase {needle haystack} {
   if {$needle eq ""} {return -1}
   if {$haystack eq ""} {return -1}
@@ -78,10 +142,14 @@ if $ddebug {set dbg_log {}}
 
 proc do_debug {s} {
   if $::ddebug {
-    puts stderr $s
+    if {$::tcl_platform(platform) ne "windows"} {puts stderr $s}
     # On windows, stderr output goes nowhere.
     # Therefore also debug output for the log dialog.
     lappend ::dbg_log $s
+    file mkdir ${::instroot}/temp
+    set dbg [open "${::instroot}/temp/mydbglog" a]
+    puts $dbg "TCL: $s"
+    chan close $dbg
     # Track debug output in the log dialog if it is running:
     if [winfo exists .tllg.dbg.tx] {
       .tllg.dbg.tx configure -state normal
@@ -103,7 +171,7 @@ proc maketemp {ext} {
     # create empty file. although we just want a name,
     # we must make sure that it can be created.
     set fid [open $fname w]
-    close $fid
+    chan close $fid
     if {! [file exists $fname]} {error "Cannot create temporary file"}
     if {$::tcl_platform(platform) eq "unix"} {
       file attributes $fname -permissions 0600
@@ -167,10 +235,10 @@ proc long_message {str type {p "."}} {
   }
   if [winfo exists .tlmg.cancel] {
     bind .tlmg <Escape> {.tlmg.cancel invoke}
-    wm protocol WM_DELETE_WINDOW {cancel_or_destroy .tlmg.cancel .tlmg}
+    wm protocol .tlmg WM_DELETE_WINDOW {cancel_or_destroy .tlmg.cancel .tlmg}
   } elseif {$type eq "ok"} {
     bind .tlmg <Escape> {.tlmg.ok invoke}
-    wm protocol WM_DELETE_WINDOW {cancel_or_destroy .tlmg.ok .tlmg}
+    wm protocol .tlmg WM_DELETE_WINDOW {cancel_or_destroy .tlmg.ok .tlmg}
   }
 
   ppack [ttk::frame .tlmg.tx] -in .tlmg.bg -side top -fill both -expand 1
@@ -206,7 +274,44 @@ proc any_message {str type {p "."}} {
   }
 } ; # any_message
 
-### enabling and disabling user interaction
+##### tl global data ##################################################
+
+set ::last_cmd ""
+
+set ::progname [info script]
+regexp {^.*[\\/]([^\\/\.]*)(?:\....)?$} $progname dummy ::progname
+set ::procid [pid]
+
+# package repositories
+array unset ::repos
+
+# mirrors: dict of dicts of lists of urls per country per continent
+# moved to tltcl.tcl
+#set ::mirrors [dict create]
+
+# dict of (local and global) package dicts
+set ::pkgs [dict create]
+
+# platforms
+set ::platforms [dict create]
+
+set ::have_remote 0 ; # remote packages info not yet loaded
+set ::need_update_tlmgr 0
+set ::n_updates 0
+set ::tlshell_updatable 0
+
+## package data to be displayed ##
+
+# sorted display data for packages; package data stored as lists
+set ::filtered [dict create]
+
+# selecting packages for display: status and detail
+set ::stat_opt "inst"
+set ::dtl_opt "all"
+# searching packages for display; also search short descriptions?
+set ::search_desc 0
+
+### enabling and disabling user interaction based on global data
 
 proc selective_dis_enable {} {
   # disable actions which make no sense at the time
@@ -276,43 +381,6 @@ proc total_dis_enable {y_n} {
   }
 } ; # total_dis_enable
 
-##### tl global data ##################################################
-
-set ::last_cmd ""
-
-set ::progname [info script]
-regexp {^.*[\\/]([^\\/\.]*)(?:\....)?$} $progname dummy ::progname
-set ::procid [pid]
-
-# package repositories
-array unset ::repos
-
-# mirrors: dict of dicts of lists of urls per country per continent
-# moved to tltcl.tcl
-#set ::mirrors [dict create]
-
-# dict of (local and global) package dicts
-set ::pkgs [dict create]
-
-# platforms
-set ::platforms [dict create]
-
-set ::have_remote 0 ; # remote packages info not yet loaded
-set ::need_update_tlmgr 0
-set ::n_updates 0
-set ::tlshell_updatable 0
-
-## package data to be displayed ##
-
-# sorted display data for packages; package data stored as lists
-set ::filtered [dict create]
-
-# selecting packages for display: status and detail
-set ::stat_opt "inst"
-set ::dtl_opt "all"
-# searching packages for display; also search short descriptions?
-set ::search_desc 0
-
 ##### handling tlmgr via pipe and stderr tempfile #####################
 
 set ::prmpt "tlmgr>"
@@ -328,20 +396,22 @@ set ::show_output 0
 # EOF is indicated by a return value of -1.
 
 proc read_err_tempfile {} {
-  set len 0
-  while 1 {
-    set len [chan gets $::err l]
-    if {$len >= 0} {
-      lappend ::err_log $l
-    } else {
-      break
+  if [info exists ::err] {
+    set len 0
+    while 1 {
+      set len [chan gets $::err l]
+      if {$len >= 0} {
+        lappend ::err_log $l
+      } else {
+        break
+      }
     }
   }
 } ; # read_err_tempfile
 
 proc err_exit {{m ""}} {
   do_debug "error exit"
-  if [info exists ::err] read_err_tempfile
+  read_err_tempfile
   if {$m ne ""} {
     set ::err_log [linsert $::err_log 0 $m]
   }
@@ -358,6 +428,12 @@ proc start_tlmgr {{args ""}} {
   # to process initial tlmgr output before continuing.
   unset -nocomplain ::done_waiting
   do_debug "opening tlmgr"
+  # -gui -. -gui-lang
+  for {set i 0} {$i < [llength $args]} {incr i} {
+    if {[lindex $args $i] eq "-lang"} {
+      set args [lreplace $args $i $i "-gui-lang"]
+    }
+  }
   set cmd [list "|tlmgr" {*}$args "--machine-readable" "shell" 2>>$::err_file]
   if [catch {open $cmd w+} ::tlshl] {
     tk_messageBox -message [get_stacktrace]
@@ -532,8 +608,8 @@ proc run_cmd_waiting {cmd} {
 # with a virtual repository, which is the combined set of repositories,
 # with pinning applied if there is more than one repository.
 # But get_packages_info_remote must invoke
-# show_repositories to display updated verification info.
-# show_repositories is also invoked by initialize.
+# show_repos to display updated verification info.
+# show_repos is also invoked by initialize.
 
 # get_packages_info_local is invoked only once, at initialization.  After
 # installations and removals, the collected information is updated by
@@ -565,6 +641,18 @@ proc is_updatable {nm} {
   return [expr {$lr > 0 && $rr > 0 && $rr > $lr}]
 }
 
+proc display_updated_globals {} {
+  if {$::have_remote && $::need_update_tlmgr} {
+    .topfll.luptodate configure -text [__ "Needs updating"]
+  } elseif $::have_remote {
+    .topfll.luptodate configure -text [__ "Up to date"]
+  } else {
+    .topfll.luptodate configure -text [__ "Unknown"]
+  }
+  # ... and status of update buttons
+  selective_dis_enable
+}
+
 proc update_globals {} {
   if {! $::have_remote} return
   set ::n_updates 0
@@ -575,15 +663,7 @@ proc update_globals {} {
   set ::tlshell_updatable [is_updatable tlshell]
 
   # also update displayed status info
-  if {$::have_remote && $::need_update_tlmgr} {
-    .topfll.luptodate configure -text [__ "Needs updating"]
-  } elseif $::have_remote {
-    .topfll.luptodate configure -text [__ "Up to date"]
-  } else {
-    .topfll.luptodate configure -text [__ "Unknown"]
-  }
-  # ... and status of update buttons
-  selective_dis_enable
+  display_updated_globals
 }
 
 # The package display treeview widget in the main window has columns
@@ -675,25 +755,26 @@ proc collect_filtered {} {
     dict lappend ::filtered $nm $v
     dict lappend ::filtered $nm [dict get $pk shortdesc]
   }
-  display_packages_info
 } ; # collect_filtered
 
-# derive the set of platforms from the dictionary of packages:
-# collect the values $plname from packages 'tex\.$plname'
+proc collect_and_display_filtered {} {
+  collect_filtered
+  display_packages_info
+}
+
 proc get_platforms {} {
   # guarantee fresh start
   foreach k $::platforms {dict unset ::platforms $k}
   set ::platforms [dict create]
-  # glob-style matching: $k should start with "tex."
-  foreach k [dict keys $::pkgs "tex.*"] {
-    set plname [string range $k 4 end]
-    if {$plname eq ""} continue
-    set pl [dict create "cur" 0 "fut" 0]
-    if {[dict get $::pkgs $k "localrev"] > 0} {
-      dict set pl "cur" 1
-      dict set pl "fut" 1
+  run_cmd_waiting "platform list"
+  foreach l $::out_log {
+    if [regexp {^\(i\)\s+(\S+)\s*$} $l dum plname] {
+      set pl [dict create "cur" 1 "fut" 1]
+      dict set ::platforms $plname $pl
+    } elseif [regexp {^\s+(\S+)\s*$} $l dum plname] {
+      set pl [dict create "cur" 0 "fut" 0]
+      dict set ::platforms $plname $pl
     }
-    dict set ::platforms $plname $pl
   }
 }
 
@@ -749,13 +830,11 @@ proc abort_load {} {
     catch {exec -ignorestderr taskkill /pid [pid] /t /f}
   } else {
     catch {exec -ignorestderr kill -9 [pid]}
-    # kill -9 should also be ok for darwin
   }
 } ; # abort load
 
-# activate abort button
 # toplevel with abort button in case loading of a repository takes too long.
-# it should disappear if loading finishes
+# it can be closed when loading finishes
 proc splash_loading {} {
 
   create_dlg .loading .
@@ -765,37 +844,23 @@ proc splash_loading {} {
   # wallpaper
   pack [ttk::frame .loading.bg -padding 3] -fill both -expand 1
 
-  set ::do_track_loading \
-      [expr {[dict get $::pkgs texlive.infra localrev] >= 51676}]
-
-  if $::do_track_loading {
-    set lbl [__ \
-                 "If loading takes too long, press Abort and choose another repository."]
-  } else {
-    set lbl [__ "Trying to load %s.
-
-If this takes too long, press Abort and choose another repository." \
-              $::repos(main)]
-  }
+  set lbl [__ \
+       "If loading takes too long, press Abort and choose another repository."]
   append lbl "\n([__ "Options"] \/ [__ "Repositories"] ...)"
   ppack [ttk::label .loading.l0 -text $lbl \
-             -wraplength [expr {60*$::cw}] -justify left] \
-      -in .loading.bg -anchor w
+           -wraplength [expr {60*$::cw}] -justify left] \
+    -in .loading.bg -anchor w
 
-  if $::do_track_loading {
-    pack [ttk::frame .loading.tfr] -in .loading.bg -expand 1 -fill x
-    pack [ttk::scrollbar .loading.scroll -command ".loading.tx yview"] \
-        -in .loading.tfr -side right -fill y
-    ppack [text .loading.tx -height 5 -wrap word \
-              -yscrollcommand ".loading.scroll set"] \
-        -in .loading.tfr -expand 1 -fill both
-  }
+  pack [ttk::frame .loading.tfr] -in .loading.bg -expand 1 -fill x
+  pack [ttk::scrollbar .loading.scroll -command ".loading.tx yview"] \
+      -in .loading.tfr -side right -fill y
+  ppack [text .loading.tx -height 5 -wrap word \
+            -yscrollcommand ".loading.scroll set"] \
+      -in .loading.tfr -expand 1 -fill both
   pack [ttk::frame .loading.buttons] -in .loading.bg -expand 1 -fill x
-  if $::do_track_loading {
-    ttk::button .loading.close -text [__ "Close"] -command {end_dlg "" .loading}
-    ppack .loading.close -in .loading.buttons -side right
-    .loading.close configure -state disabled
-  }
+  ttk::button .loading.close -text [__ "Close"] -command {end_dlg "" .loading}
+  ppack .loading.close -in .loading.buttons -side right
+  .loading.close configure -state disabled
   ttk::button .loading.abo -text [__ "Abort"] -command abort_load
   ppack .loading.abo -in .loading.buttons -side right
   wm protocol .loading {cancel_or_destroy .loading.abo .loading}
@@ -803,22 +868,24 @@ If this takes too long, press Abort and choose another repository." \
   place_dlg .loading .
 } ; # splash_loading
 
+
 proc track_err {} {
-  if $::do_track_loading {
-    set inx0 [llength $::err_log]
-    #puts stderr "track_err: $inx0"
-    read_err_tempfile
+  set inx0 [llength $::err_log]
+  read_err_tempfile
+  .loading.tx configure -state normal
+  for {set i $inx0} {$i < [llength $::err_log]} {incr i} {
+    .loading.tx insert end "[lindex $::err_log $i]\n"
+  }
+  .loading.tx configure -state disabled
+  update idletasks
+  if {![info exists ::loaded]} {
+    after 500 track_err
+  } else {
+    .loading.close state !disabled
+    .loading.abo state disabled
     .loading.tx configure -state normal
-    for {set i $inx0} {$i < [llength $::err_log]} {incr i} {
-      .loading.tx insert end "[lindex $::err_log $i]\n"
-    }
+    .loading.tx insert end [__ "Done loading"]
     .loading.tx configure -state disabled
-    if {![info exists ::loaded]} {
-      after 500 track_err
-    } else {
-      .loading.close state !disabled
-      .loading.abo state disabled
-    }
   }
 }
 
@@ -837,7 +904,7 @@ proc get_packages_info_remote {} {
   splash_loading
 
   unset -nocomplain ::loaded
-  track_err ; # is a no-op unless $::do_track_loading
+  track_err
   if [catch {run_cmd \
     "info --data name,localrev,remoterev,cat-version,category,shortdesc"}] {
     do_debug [get_stacktrace]
@@ -846,9 +913,6 @@ proc get_packages_info_remote {} {
   }
   vwait ::done_waiting
   set ::loaded 1
-  if {! $::do_track_loading} {
-    destroy .loading
-  } ; # otherwise, .loading destroyed by close button of track_err
   set re {^([^,]+),([0-9]+),([0-9]+),([^,]*),([^,]*),(.*)$}
   foreach l $::out_log {
     if [regexp $re $l m nm lrev rrev rcatv catg pdescr] {
@@ -1074,6 +1138,9 @@ proc set_repos_in_tlmgr {} {
 }; # set_repos_in_tlmgr
 
 proc show_repos {} {
+  # this proc lists the configured repository/ies
+  # in the upper left portion of the main window.
+  # it makes one call to the back end, but no need to go online.
   set w .toprepo
   foreach ch [winfo children $w] {destroy $ch}
   set nms [array names ::repos]
@@ -1136,10 +1203,11 @@ proc show_repos {} {
 proc repos_commit {} {
   set changes 0
   # set repositories then add pinning if appropriate
-  if {! [regexp {^\s*$} [.tlr.new get]]} {
-    # repository entry widget non-empty: retrieve it
-    if {$::repos(main) ne [.tlr.new get]} {
-      set ::repos(main) [.tlr.new get]
+  set new_repo [forward_slashify [.tlr.new get]]
+  if {! [regexp {^\s*$} $new_repo]} {
+    # repository entry widget non-empty: use it
+    if {$::repos(main) ne $new_repo} {
+      set ::repos(main) $new_repo
       set changes 1
     }
   }
@@ -1183,7 +1251,7 @@ proc save_load_repo {} {
   # reload remote package information
   set ::have_remote 0
   get_packages_info_remote
-  collect_filtered
+  collect_and_display_filtered
 }
 
 proc select_mir {m} {
@@ -1341,7 +1409,7 @@ if {$::tcl_platform(platform) ne "windows"} {
     run_cmds $cmds 1
     vwait ::done_waiting
     update_local_revnumbers
-    collect_filtered
+    collect_and_display_filtered
 
   } ; # platforms_do
 
@@ -1423,7 +1491,7 @@ proc finish_restore {} {
   # We won't wait for the log dialog to close, but we will
   # update the packages display in the main window.
   update_local_revnumbers
-  collect_filtered
+  collect_and_display_filtered
 } ; # finish_restore
 
 proc restore_all {} {
@@ -1587,16 +1655,16 @@ proc restore_backups_dialog {} {
 
 ##### package-related #####
 
-proc update_self_w32 {} {
-  if $::multiuser {
-    set mess \
-        [__ "Close this shell and run in an administrative command-prompt:"]
-  } else {
-    set mess [__ "Close this shell and run in a command-prompt:"]
-  }
-  set mess [string cat $mess "\n\ntlmgr update --self"]
-  tk_messageBox -message $mess
-  return
+### updating
+
+proc update_tlmgr_w32 {} {
+  close_tlmgr
+  # don't try pipes or capturing, because of
+  # tlmgr's acrobatics with nested command prompts
+  wm iconify .
+  exec -ignorestderr cmd /k "start cmd /k tlmgr update --self"
+  exec $::progname &
+  destroy .
 }
 
 proc update_tlmgr {} {
@@ -1605,7 +1673,7 @@ proc update_tlmgr {} {
     return
   }
   if {$::tcl_platform(platform) eq "windows"} {
-    update_self_w32
+    update_tlmgr_w32
     return
   }
   run_cmd "update --self" 1
@@ -1614,7 +1682,7 @@ proc update_tlmgr {} {
   update_local_revnumbers
   .topfr.linfra configure -text \
       "tlmgr: r[dict get $::pkgs texlive.infra localrev]"
-  collect_filtered
+  collect_and_display_filtered
 } ; # update_tlmgr
 
 proc update_all {} {
@@ -1640,7 +1708,7 @@ proc update_all {} {
     vwait ::done_waiting
     update_local_revnumbers
   }
-  collect_filtered
+  collect_and_display_filtered
 } ; # update_all
 
 ### doing something with some packages
@@ -1700,7 +1768,7 @@ proc install_pkgs {sel_opt {pk ""}} {
   }
   update_local_revnumbers
   if {$sel_opt eq "marked"} {mark_all 0}
-  collect_filtered
+  collect_and_display_filtered
 } ; # install_pkgs
 
 proc update_pkgs {sel_opt {pk ""}} {
@@ -1773,7 +1841,7 @@ proc update_pkgs {sel_opt {pk ""}} {
   }
   update_local_revnumbers
   if {$sel_opt eq "marked"} {mark_all 0}
-  collect_filtered
+  collect_and_display_filtered
 } ; # update_pkgs
 
 proc remove_pkgs {sel_opt {pk ""}} {
@@ -1834,7 +1902,7 @@ proc remove_pkgs {sel_opt {pk ""}} {
   }
   update_local_revnumbers
   if {$sel_opt eq "marked"} {mark_all 0}
-  collect_filtered
+  collect_and_display_filtered
 } ; # remove_pkgs
 
 # restoring packages is a rather different story, controlled by the
@@ -1920,55 +1988,39 @@ proc set_paper {p} {
   run_cmd "paper paper $p" 1
 }
 
-proc set_language_no_restart {l} {
-  set ok 1
-  if [catch {exec kpsewhich -var-value "TEXMFCONFIG"} d] {set ok 0}
-  if $ok {
-    set d [file join $d "tlmgr"]
-    if [catch {file mkdir $d}] {set ok 0}
-  }
-  set fn [file join $d "config"]
-  set oldlines [list]
-  if {$ok && ! [catch {open $fn r} fid]} {
-    set cnt 0
-    while 1 {
-      if [catch {chan gets $fid} ll] break
-      if [chan eof $fid] break
-      incr cnt
-      if {! [regexp {^\s*gui-lang} $ll]} {
-          lappend oldlines $ll
-      }
-      if {$cnt>20} break
-    }
-    catch {chan close $fid}
-  }
-  lappend oldlines "gui-lang = $l"
-  if {$ok && ! [catch {open $fn w} fid]} {
-    foreach ll $oldlines {
-      if [catch {puts $fid $ll}] {
-        set ok 0
-        break
-      }
-    }
-    catch {chan close $fid}
-  }
-  return $ok
-} ; # set_language_no_restart
+#### gui options ####
 
 proc set_language {l} {
-  if [set_language_no_restart $l] {
-    restart_self
-  } else {
-    tk_messageBox -message [__ "Cannot set default GUI language"] -icon error
+  set ::lang $l
+  load_translations
+  rebuild_interface
+  if {! [save_config_var "gui-lang" $::lang]} {
+    tk_messageBox -message [__"Cannot save language setting"]
   }
 }
 
+proc set_fontscale {s} {
+  set ::tkfontscale $s
+  redo_fonts
+  rebuild_interface
+  if {[dict get $::pkgs texlive.infra localrev] >= 54766} {
+    if {! [save_config_var "tkfontscale" $::tkfontscale]} {
+      tk_messageBox -message [__"Cannot save font scale setting"]
+    }
+  }
+}
+
+proc zoom {n} {
+  if {$n <= 0} {set n 1}
+  set_fontscale [expr {$n*$::tkfontscale}]
+}
+
 ##### running external commands #####
 
 # For capturing an external command, we need a separate output channel,
 # but we reuse ::out_log.
 # stderr is bundled with stdout so ::err_log should stay empty.
-proc read_capt {} {
+proc read_capture {} {
   set l "" ; # will contain the line to be read
   if {([catch {chan gets $::capt l} len] || [chan eof $::capt])} {
     catch {chan close $::capt}
@@ -1978,7 +2030,7 @@ proc read_capt {} {
     lappend ::out_log $l
     log_widget_add $l
   }
-}; # read_capt
+}; # read_capture
 
 proc run_external {cmd mess} {
   set ::out_log {}
@@ -1994,10 +2046,16 @@ proc run_external {cmd mess} {
     tk_messageBox -message "Failure to launch $cmd"
   }
   chan configure $::capt -buffering line -blocking 0
-  chan event $::capt readable read_capt
+  chan event $::capt readable read_capture
   log_widget_init
 }
 
+proc about_cmd {} {
+  set msg "\u00a9 2017-2020 Siep Kroonenberg\n\n"
+  append msg [__ "GUI interface for TeX Live Manager\nImplemented in Tcl/Tk"]
+  tk_messageBox -message $msg
+}
+
 proc show_help {} {
   set ::env(NOPERLDOC) 1
   long_message [exec tlmgr --help] ok
@@ -2024,7 +2082,7 @@ proc show_help {} {
 proc try_loading_remote {} {
   if {[possible_repository $::repos(main)]} {
     get_packages_info_remote
-    collect_filtered
+    collect_and_display_filtered
   } else {
     set mes [__ "%s is not a local or remote repository.
 Please configure a valid repository" $::repos(main)]
@@ -2039,9 +2097,6 @@ proc populate_main {} {
 
   wm title . "TeX Live Shell"
 
-  # width of '0', as a rough estimate of average character width
-  set ::cw [font measure TkTextFont "0"]
-
   ## menu ##
 
   # dummy empty menu to replace the real menu .mn in disabled states.
@@ -2061,10 +2116,10 @@ proc populate_main {} {
     .mn configure -borderwidth 1
     .mn configure -background $::default_bg
 
-    # plain_unix: avoid a RenderBadPicture error on quitting.
-    # 'send' changes the shutdown sequence,
-    # which avoids triggering the bug.
-    # 'tk appname <something>' restores 'send' and avoids the bug
+    # plain_unix: avoid a possible RenderBadPicture error on quitting
+    # when there is a menu.
+    # 'send' bypasses the bug by changing the shutdown sequence.
+    # 'tk appname <something>' restores 'send'.
     bind . <Destroy> {
       catch {tk appname appname}
     }
@@ -2111,18 +2166,45 @@ proc populate_main {} {
 
   if {[llength $::langs] > 1} {
     incr inx
-    .mn.opt add cascade -label [__ "GUI language (restarts tlshell)"] \
+    .mn.opt add cascade -label [__ "GUI language"] \
         -menu .mn.opt.lang
     menu .mn.opt.lang
     foreach l [lsort $::langs] {
       if {$l eq $::lang} {
-        .mn.opt.lang add command -label "$l *"
+        set mlabel "$l *"
       } else {
-        .mn.opt.lang add command -label "$l" -command "set_language $l"
+        set mlabel $l
       }
+      .mn.opt.lang add command -label $mlabel \
+          -command "set_language $l"
     }
   }
 
+  incr inx
+  .mn.opt add cascade -label [__ "GUI font scaling"] \
+      -menu .mn.opt.fscale
+  menu .mn.opt.fscale
+  .mn.opt.fscale add command -label \
+      "[__ "Current"]:  [format {%.2f} $::tkfontscale]"
+  foreach s {0.6 0.8 1 1.2 1.6 2 2.5 3 3.8 5 6 7.5 9} {
+    .mn.opt.fscale add command -label $s -command "set_fontscale $s"
+  }
+
+  # browser-style keyboard shortcuts for scaling
+  bind . <Control-KeyRelease-minus> {zoom 0.8}
+  bind . <Control-KeyRelease-equal> {zoom 1.25}
+  bind . <Control-Shift-KeyRelease-equal> {zoom 1.25}
+  bind . <Control-KeyRelease-plus> {zoom 1.25}
+  bind . <Control-KeyRelease-0> {set_fontscale 1}
+  if {$::tcl_platform(os) eq "Darwin"} {
+    bind . <Command-KeyRelease-minus> {zoom 0.8}
+    bind . <Command-KeyRelease-equal> {zoom 1.25}
+    bind . <Command-Shift-KeyRelease-equal> {zoom 1.25}
+    bind . <Command-KeyRelease-plus> {zoom 1.25}
+    bind . <Command-KeyRelease-0> {set_fontscale 1}
+  }
+
+
   if {$::tcl_platform(platform) ne "windows"} {
     incr inx
     set ::inx_platforms $inx
@@ -2131,10 +2213,7 @@ proc populate_main {} {
 
   .mn add cascade -label [__ "Help"] -menu .mn.help -underline 0
   menu .mn.help
-  .mn.help add command -label [__ "About"] -command {
-    tk_messageBox -message [string cat "\u00a9 2017-2019 Siep Kroonenberg
-
-" [__ "GUI interface for TeX Live Manager\nImplemented in Tcl/Tk"]]}
+  .mn.help add command -label [__ "About"] -command about_cmd
   .mn.help add command -label [__ "tlmgr help"] -command show_help
 
   ## menu end
@@ -2165,6 +2244,13 @@ proc populate_main {} {
   # top of main window
   ppack [ttk::frame .topf] -in .bg -side top -anchor w -fill x
 
+  if $::ddebug {
+    ppack [ttk::label .topf.test -text [info nameofexecutable]]
+    ppack [ttk::label .topf.test2 -text $::progname]
+    ppack [ttk::label .topf.test3 -text \
+           [string range $::env(PATH) 0 59]]
+  }
+
   # left frame
   pack [ttk::frame .topfl] -in .topf -side left -anchor nw
 
@@ -2210,16 +2296,16 @@ proc populate_main {} {
   # filter on status: inst, all, upd
   ttk::label .pkfilter.lstat -font TkHeadingFont -text [__ "Status"]
   ttk::radiobutton .pkfilter.inst -text [__ "Installed"] -value inst \
-      -variable ::stat_opt -command collect_filtered
+      -variable ::stat_opt -command collect_and_display_filtered
   ttk::radiobutton .pkfilter.alls -text [__ "All"] -value all \
       -variable ::stat_opt -command {
         if {! $::have_remote} get_packages_info_remote
-        collect_filtered
+        collect_and_display_filtered
       }
   ttk::radiobutton .pkfilter.upd -text [__ "Updatable"] -value upd \
       -variable ::stat_opt -command {
         if {! $::have_remote} get_packages_info_remote
-        collect_filtered
+        collect_and_display_filtered
       }
   grid .pkfilter.lstat -column 0 -row 0 -sticky w -padx {3 50}
   pgrid .pkfilter.inst -column 0 -row 1 -sticky w
@@ -2229,11 +2315,11 @@ proc populate_main {} {
   # filter on detail level: all, coll, schm
   ttk::label .pkfilter.ldtl -font TkHeadingFont -text [__ "Detail >> Global"]
   ttk::radiobutton .pkfilter.alld -text [__ All] -value all \
-      -variable ::dtl_opt -command collect_filtered
+      -variable ::dtl_opt -command collect_and_display_filtered
   ttk::radiobutton .pkfilter.coll -text [__ "Collections and schemes"] \
-      -value coll -variable ::dtl_opt -command collect_filtered
+      -value coll -variable ::dtl_opt -command collect_and_display_filtered
   ttk::radiobutton .pkfilter.schm -text [__ "Only schemes"] -value schm \
-      -variable ::dtl_opt -command collect_filtered
+      -variable ::dtl_opt -command collect_and_display_filtered
   pgrid .pkfilter.ldtl -column 1 -row 0 -sticky w
   pgrid .pkfilter.alld -column 1 -row 1 -sticky w
   pgrid .pkfilter.coll -column 1 -row 2 -sticky w
@@ -2323,6 +2409,30 @@ proc populate_main {} {
   wm protocol . WM_DELETE_WINDOW {cancel_or_destroy .q .}
   wm resizable . 1 1
   wm state . normal
+  raise .
+}
+
+# to be invoked at initialization and after a font scaling change
+proc rebuild_interface {} {
+  foreach c [winfo children .] {catch {destroy $c}}
+
+  # for busy/idle indicators
+  set ::busy [__ "Idle"]
+  populate_main
+  # and now redisplay all data
+  if {$::tcl_platform(platform) eq "windows"} {
+    .topfr.ladmin configure -text \
+        [expr {$::multiuser ? [__ "Multi-user"] : [__ "Single-user"]}]
+  }
+  # svns for  tlmgr and tlshell
+  .topfr.linfra configure -text \
+      "tlmgr: r[dict get $::pkgs texlive.infra localrev]"
+  .topfr.lshell configure -text \
+      "tlshell: r[dict get $::pkgs tlshell localrev]"
+  show_repos
+  display_packages_info
+  display_updated_globals
+  if {$::tcl_platform(platform) eq "windows"} {wm deiconify .}
 }
 
 ##### initialize ######################################################
@@ -2372,15 +2482,8 @@ proc initialize {} {
     set ::flid [open $fname w]
   }
 
-  # languages
-  set ::langs [list "en"]
-  foreach l [glob -nocomplain -directory \
-                 [file join $::instroot "tlpkg" "translations"] *.po] {
-    lappend ::langs [string range [file tail $l] 0 end-3]
-  }
-
   # store language in tlmgr configuration
-  set_language_no_restart $::lang
+  save_config_var "gui-lang" $::lang
 
   # in case we are going to do something with json:
   # add json subdirectory to auto_path, but at low priority
@@ -2392,33 +2495,24 @@ proc initialize {} {
   populate_main
 
   # testing writablilty earlier led to sizing problems
-  if {! [file writable $::instroot]} {
+  if {! [dir_writable $::instroot]} {
     set ans [tk_messageBox -type yesno -icon warning -message \
          [__ "%s is not writable. You can probably not do much.
   Are you sure you want to continue?" $::instroot]]
     if {$ans ne "yes"} {exit}
   }
 
-  start_tlmgr
+  start_tlmgr {*}$::argv
   if {$::tcl_platform(platform) eq "windows"} {
     run_cmd_waiting "option multiuser"
     set ::multiuser 0
     foreach l $::out_log {
       if [regexp {^\s*multiuser\s+([01])\s*$} $l d ::multiuser] break
     }
-    .topfr.ladmin configure -text \
-        [expr {$::multiuser ? [__ "Multi-user"] : [__ "Single-user"]}]
   }
   get_packages_info_local
+  collect_filtered
   get_repos_from_tlmgr
-  show_repos
-  # svns for  tlmgr and tlshell
-  .topfr.linfra configure -text \
-      "tlmgr: r[dict get $::pkgs texlive.infra localrev]"
-  .topfr.lshell configure -text \
-      "tlshell: r[dict get $::pkgs tlshell localrev]"
-  collect_filtered ; # invokes display_packages_info
-  selective_dis_enable
 }; # initialize
 
 initialize
