@@ -8,7 +8,7 @@
 -----------------------------------------------------------------------
 
         xindex = xindex or { }
- local version = 0.18
+ local version = 0.23
 xindex.version = version
 --xindex.self = "xindex"
 
@@ -30,7 +30,7 @@ Report bugs to
 kpse.set_program_name("luatex")
 
 local f = kpse.find_file("lualibs.lua")
-print ("Dateiname "..f)
+print ("filename "..f)
 require("lualibs")  -- all part of LuaTeX
 require('unicode')
 require('string')
@@ -49,6 +49,7 @@ local args = require ('xindex-lapp') [[
     -o,--output (default "")
     -l,--language (default en)
     -p,--prefix (default L)
+    -u,--use_UCA
     <input> (string)
 ]]
 
@@ -174,12 +175,55 @@ escape_chars = { -- by default " is the escape char
   {esc_char..')', '//escapedparenright//',')'    }
 }
 
-language = string.lower(args["language"])
+local language = "en" -- default language
+
+language = string.lower(args["language"]):sub(1, 2)
 writeLog(2,"Language = "..language.."\n",1) 
+if (indexheader[language] == nil) then
+  writeLog(2,'Corrected the unknown language "'..language..'" to "en"'.."\n",0) 
+  language = "en"
+end  
 index_header = indexheader[language]
 if vlevel > 0 then for i=1,#index_header do writeLog(2,index_header[i].."\n",1) end end
-page_folium = folium[language]
+if (folium[language] == nil) then
+  writeLog(2,'Corrected the unknown language "'..language..'" for page folium to "en"'.."\n",0) 
+  page_folium = folium["en"]
+else
+  page_folium = folium[language]
+end  
 
+use_UCA = args["use_UCA"]
+if use_UCA then
+  writeLog(1,"Will use LUA-UCA\n",1)
+  ducet = require "lua-uca.lua-uca-ducet"
+  collator = require "lua-uca.lua-uca-collator"
+  languages = require "lua-uca.lua-uca-languages"
+  collator_obj = collator.new(ducet)
+  
+  local uca_config_file = "xindex-cfg-uca.lua"
+  writeLog(2,"Loading local UCA config file "..uca_config_file,0)
+  UCA_Config_File = kpse.find_file(uca_config_file) 
+  uca_cfg = require(UCA_Config_File)
+  writeLog(2," ... done\n",0)
+  
+-- language name specified on the command line doesn't seem to be available
+-- in the config file, so we just try to find it ourselves
+  for i, a in ipairs(arg) do
+    if a == "-l" or a=="--language" then
+      language = arg[i+1]
+      break
+    end
+  end
+
+  if languages[language] then
+    print("[Lua-UCA] Loading language: " .. language)
+    collator_obj = languages[language](collator_obj)
+  end
+else
+  writeLog(1,"Will _not_ use LUA-UCA\n",1)
+end
+
+upper = unicode.utf8.upper
 
 no_caseSensitive = args["no_casesensitive"]
 if no_caseSensitive then
@@ -206,4 +250,3 @@ dofile(BaseRunFile)
 
 logFile:close()
 
-
