@@ -25,6 +25,7 @@ local scanners    = interfaces.scanners
 local register    = interfaces.registerscanner
 
 local compile     = tokens.compile or function() end
+local presets     = tokens.presets
 
 local dummy       = function() end
 
@@ -47,6 +48,10 @@ function interfaces.implement(specification)
     if name == "" then
         name = nil
     end
+    local p = arguments and presets[arguments]
+    if p then
+        arguments = p
+    end
     local scanner
     local resetter = onlyonce and name and commands.ctxresetter(name)
     if resetter then
@@ -72,7 +77,7 @@ function interfaces.implement(specification)
     if scanners[name] and not specification.overload then
         report("warning: 'scanners.%s' is redefined",name)
     end
---     scanners[name] = scanner
+ -- scanners[name] = scanner -- we now use:
     register(name,scanner,specification.protected,specification.public,specification.call)
     if private then
         return
