@@ -12,15 +12,18 @@ return {
     ["constants"] = {
         --
         "zerocount", "minusone", "minustwo", "plusone", "plustwo", "plusthree", "plusfour", "plusfive",
-        "plussix", "plusseven", "pluseight", "plusnine", "plusten", "plussixteen", "plushundred", "plustwohundred",
+        "plussix", "plusseven", "pluseight", "plusnine", "plusten", "plussixteen",
+        "plusfifty", "plushundred", "plusonehundred", "plustwohundred", "plusfivehundred",
         "plusthousand", "plustenthousand", "plustwentythousand", "medcard", "maxcard", "maxcardminusone",
-        "zeropoint", "onepoint", "halfapoint", "onebasepoint", "maxdimen", "scaledpoint", "thousandpoint",
+        "zeropoint", "onepoint", "halfapoint", "onebasepoint", "maxcount", "maxdimen", "scaledpoint", "thousandpoint",
         "points", "halfpoint",
         "zeroskip",
         "zeromuskip", "onemuskip",
         "pluscxxvii", "pluscxxviii", "pluscclv", "pluscclvi",
         "normalpagebox",
-        --        --
+        --
+        "directionlefttoright", "directionrighttoleft",
+        --
         "endoflinetoken", "outputnewlinechar",
         --
         "emptytoks", "empty", "undefined",
@@ -29,11 +32,9 @@ return {
         --
         "bigskipamount", "medskipamount", "smallskipamount",
         --
-        "fmtname", "fmtversion", "texengine", "texenginename", "texengineversion",
+        "fmtname", "fmtversion", "texengine", "texenginename", "texengineversion", "texenginefunctionality",
         "luatexengine", "pdftexengine", "xetexengine", "unknownengine",
-     -- "etexversion",
-     -- "pdftexversion", "pdftexrevision",
-     -- "xetexversion", "xetexrevision",
+        "contextformat", "contextversion", "contextkind", "contextlmtxmode", "contextmark", "mksuffix",
         --
         "activecatcode",
         --
@@ -59,6 +60,7 @@ return {
         "spaceasciicode", "hashasciicode", "dollarasciicode", "commentasciicode", "ampersandasciicode",
         "colonasciicode", "backslashasciicode", "circumflexasciicode", "underscoreasciicode",
         "leftbraceasciicode", "barasciicode", "rightbraceasciicode", "tildeasciicode", "delasciicode",
+        "leftparentasciicode", "rightparentasciicode",
         "lessthanasciicode", "morethanasciicode", "doublecommentsignal",
         "atsignasciicode", "exclamationmarkasciicode", "questionmarkasciicode",
         "doublequoteasciicode", "singlequoteasciicode", "forwardslashasciicode",
@@ -85,6 +87,7 @@ return {
         --
         "fontslantperpoint", "fontinterwordspace", "fontinterwordstretch", "fontinterwordshrink",
         "fontexheight", "fontemwidth", "fontextraspace", "slantperpoint",
+        "mathexheight", "mathemwidth",
         "interwordspace", "interwordstretch", "interwordshrink", "exheight", "emwidth", "extraspace",
         "mathsupdisplay", "mathsupnormal", "mathsupcramped", "mathsubnormal", "mathsubcombined",  "mathaxisheight",
         "muquad",
@@ -100,7 +103,9 @@ return {
         "startcomponent", "stopcomponent", "component",
         "startproduct", "stopproduct", "product",
         "startproject", "stopproject", "project",
-        "starttext", "stoptext", "startnotext", "stopnotext","startdocument", "stopdocument", "documentvariable", "setupdocument", "presetdocument",
+        "starttext", "stoptext", "startnotext", "stopnotext",
+        "startdocument", "stopdocument", "documentvariable", "unexpandeddocumentvariable", "setupdocument", "presetdocument",
+        "doifelsedocumentvariable", "doifdocumentvariableelse", "doifdocumentvariable", "doifnotdocumentvariable",
         "startmodule", "stopmodule", "usemodule", "usetexmodule", "useluamodule","setupmodule","currentmoduleparameter","moduleparameter",
         "everystarttext", "everystoptext",
         --
@@ -123,16 +128,23 @@ return {
         --
         "luastringsep", "!!bs", "!!es",
         --
-        "lefttorightmark", "righttoleftmark",
+        "lefttorightmark", "righttoleftmark", "lrm", "rlm",
+        "bidilre", "bidirle", "bidipop", "bidilro", "bidirlo",
         --
         "breakablethinspace", "nobreakspace", "nonbreakablespace", "narrownobreakspace", "zerowidthnobreakspace",
         "ideographicspace", "ideographichalffillspace",
         "twoperemspace", "threeperemspace", "fourperemspace", "fiveperemspace", "sixperemspace",
-        "figurespace", "punctuationspace", "hairspace",
+        "figurespace", "punctuationspace", "hairspace", "enquad", "emquad",
         "zerowidthspace", "zerowidthnonjoiner", "zerowidthjoiner", "zwnj", "zwj",
-        "optionalspace", "asciispacechar",
+        "optionalspace", "asciispacechar", "softhyphen",
         --
         "Ux", "eUx", "Umathaccents",
+        --
+        "parfillleftskip", "parfillrightskip",
+        --
+        "startlmtxmode", "stoplmtxmode", "startmkivmode", "stopmkivmode",
+        --
+        "wildcardsymbol",
     },
     ["helpers"] = {
         --
@@ -149,10 +161,12 @@ return {
         "texdefinition",
         --
         "doifelsesetups", "doifsetupselse", "doifsetups", "doifnotsetups", "setup", "setups", "texsetup", "xmlsetup", "luasetup", "directsetup", "fastsetup",
+        "copysetups", "resetsetups",
         "doifelsecommandhandler", "doifcommandhandlerelse", "doifnotcommandhandler", "doifcommandhandler",
         --
         "newmode", "setmode", "resetmode",
         "newsystemmode", "setsystemmode", "resetsystemmode", "pushsystemmode", "popsystemmode",
+        "globalsetmode", "globalresetmode", "globalsetsystemmode", "globalresetsystemmode",
         "booleanmodevalue",
         --
         "newcount", "newdimen", "newskip", "newmuskip", "newbox", "newtoks", "newread", "newwrite", "newmarks", "newinsert", "newattribute", "newif",
@@ -161,14 +175,19 @@ return {
         "then",
         "begcsname",
         --
+        "autorule",
+        --
         "strippedcsname","checkedstrippedcsname",
         --
         "firstargumentfalse", "firstargumenttrue",
         "secondargumentfalse", "secondargumenttrue",
         "thirdargumentfalse", "thirdargumenttrue",
         "fourthargumentfalse", "fourthargumenttrue",
-        "fifthargumentfalse", "fifthsargumenttrue",
-        "sixthargumentfalse", "sixtsargumenttrue",
+        "fifthargumentfalse", "fifthargumenttrue",
+        "sixthargumentfalse", "sixthargumenttrue",
+        "seventhargumentfalse", "seventhargumenttrue",
+        --
+        "vkern", "hkern",
         --
         "doglobal", "dodoglobal", "redoglobal", "resetglobal",
         --
@@ -176,7 +195,7 @@ return {
         --
         "donetrue", "donefalse", "foundtrue", "foundfalse",
         --
-        "inlineordisplaymath","indisplaymath","forcedisplaymath","startforceddisplaymath","stopforceddisplaymath","reqno",
+        "inlineordisplaymath","indisplaymath","forcedisplaymath","startforceddisplaymath","stopforceddisplaymath","startpickupmath","stoppickupmath","reqno",
         --
         "mathortext",
         --
@@ -193,8 +212,8 @@ return {
         --
         "hglue", "vglue", "hfillneg", "vfillneg", "hfilllneg", "vfilllneg",
         --
-        "ruledhss", "ruledhfil", "ruledhfill", "ruledhfilneg", "ruledhfillneg", "normalhfillneg",
-        "ruledvss", "ruledvfil", "ruledvfill", "ruledvfilneg", "ruledvfillneg", "normalvfillneg",
+        "ruledhss", "ruledhfil", "ruledhfill", "ruledhfilll", "ruledhfilneg", "ruledhfillneg", "normalhfillneg",  "normalhfilllneg",
+        "ruledvss", "ruledvfil", "ruledvfill", "ruledvfilll", "ruledvfilneg", "ruledvfillneg", "normalvfillneg",  "normalvfilllneg",
         "ruledhbox", "ruledvbox", "ruledvtop", "ruledvcenter", "ruledmbox",
         "ruledhpack", "ruledvpack", "ruledtpack",
         "ruledhskip", "ruledvskip", "ruledkern", "ruledmskip", "ruledmkern",
@@ -203,20 +222,31 @@ return {
         --
         "filledhboxb", "filledhboxr", "filledhboxg", "filledhboxc", "filledhboxm", "filledhboxy", "filledhboxk",
         --
-        "scratchcounter", "globalscratchcounter",
-        "scratchdimen", "globalscratchdimen",
-        "scratchskip", "globalscratchskip",
-        "scratchmuskip", "globalscratchmuskip",
-        "scratchtoks", "globalscratchtoks",
-        "scratchbox", "globalscratchbox",
+        "scratchcounter", "globalscratchcounter", "privatescratchcounter",
+        "scratchdimen", "globalscratchdimen", "privatescratchdimen",
+        "scratchskip", "globalscratchskip", "privatescratchskip",
+        "scratchmuskip", "globalscratchmuskip", "privatescratchmuskip",
+        "scratchtoks", "globalscratchtoks", "privatescratchtoks",
+        "scratchbox", "globalscratchbox", "privatescratchbox",
+        --
+        "globalscratchcounterone", "globalscratchcountertwo", "globalscratchcounterthree",
+        --
+        "groupedcommand", "groupedcommandcs",
+        "triggergroupedcommand", "triggergroupedcommandcs",
+        "simplegroupedcommand", "pickupgroupedcommand",
         --
         "normalbaselineskip", "normallineskip", "normallineskiplimit",
         --
         "availablehsize", "localhsize", "setlocalhsize", "distributedhsize", "hsizefraction",
         --
-        "nextbox", "dowithnextbox", "dowithnextboxcs", "dowithnextboxcontent", "dowithnextboxcontentcs", "flushnextbox",
+        "next", "nexttoken",
         --
-        "scratchwidth", "scratchheight", "scratchdepth", "scratchoffset", "scratchdistance",
+        "nextbox", "dowithnextbox", "dowithnextboxcs", "dowithnextboxcontent", "dowithnextboxcontentcs", "flushnextbox",
+        "boxisempty",
+        --
+        "givenwidth", "givenheight", "givendepth", "scangivendimensions",
+        --
+        "scratchwidth", "scratchheight", "scratchdepth", "scratchoffset", "scratchdistance", "scratchtotal",
         "scratchhsize", "scratchvsize",
         "scratchxoffset", "scratchyoffset", "scratchhoffset", "scratchvoffset",
         "scratchxposition", "scratchyposition",
@@ -235,12 +265,15 @@ return {
         "scratchleftskip", "scratchrightskip", "scratchtopskip", "scratchbottomskip",
         --
         "doif", "doifnot", "doifelse",
+        "firstinset",
         "doifinset", "doifnotinset",
         "doifelseinset", "doifinsetelse",
         "doifelsenextchar", "doifnextcharelse",
+        "doifelsenextcharcs", "doifnextcharcselse",
         "doifelsenextoptional", "doifnextoptionalelse",
         "doifelsenextoptionalcs", "doifnextoptionalcselse",
         "doifelsefastoptionalcheck", "doiffastoptionalcheckelse",
+        "doifelsefastoptionalcheckcs", "doiffastoptionalcheckcselse",
         "doifelsenextbgroup", "doifnextbgroupelse",
         "doifelsenextbgroupcs", "doifnextbgroupcselse",
         "doifelsenextparenthesis", "doifnextparenthesiselse",
@@ -257,7 +290,11 @@ return {
         "doifelsenumber", "doifnumberelse", "doifnumber", "doifnotnumber",
         "doifelsecommon", "doifcommonelse", "doifcommon", "doifnotcommon",
         "doifinstring", "doifnotinstring", "doifelseinstring", "doifinstringelse",
-        "doifelseassignment", "doifassignmentelse", "docheckassignment",
+        "doifelseassignment", "doifassignmentelse", "docheckassignment", "doifelseassignmentcs", "doifassignmentelsecs",
+        "validassignment", "novalidassignment",
+        "doiftext", "doifelsetext", "doiftextelse", "doifnottext",
+        --
+        "quitcondition", "truecondition", "falsecondition",
         --
         "tracingall", "tracingnone", "loggingall",
         --
@@ -272,11 +309,15 @@ return {
         "singleexpandafter", "doubleexpandafter", "tripleexpandafter",
         --
         "dontleavehmode", "removelastspace", "removeunwantedspaces", "keepunwantedspaces",
-        "removepunctuation",
+        "removepunctuation", "ignoreparskip", "forcestrutdepth", "onlynonbreakablespace",
         --
         "wait", "writestatus", "define", "defineexpandable", "redefine",
         --
-        "setmeasure", "setemeasure", "setgmeasure", "setxmeasure", "definemeasure", "freezemeasure", "measure", "measured",
+        "setmeasure", "setemeasure", "setgmeasure", "setxmeasure", "definemeasure", "freezemeasure",
+        "measure", "measured", "directmeasure",
+        "setquantity", "setequantity", "setgquantity", "setxquantity", "definequantity", "freezequantity",
+        "quantity", "quantitied", "directquantity",
+     -- "quantified",
         --
         "installcorenamespace",
         --
@@ -288,7 +329,7 @@ return {
         --
         "getparameters", "geteparameters", "getgparameters", "getxparameters", "forgetparameters", "copyparameters",
         --
-        "getdummyparameters", "dummyparameter", "directdummyparameter", "setdummyparameter", "letdummyparameter",
+        "getdummyparameters", "dummyparameter", "directdummyparameter", "setdummyparameter", "letdummyparameter", "setexpandeddummyparameter",
         "usedummystyleandcolor", "usedummystyleparameter", "usedummycolorparameter",
         --
         "processcommalist", "processcommacommand", "quitcommalist", "quitprevcommalist",
@@ -310,7 +351,7 @@ return {
         "gobbleoneargument", "gobbletwoarguments", "gobblethreearguments", "gobblefourarguments", "gobblefivearguments", "gobblesixarguments", "gobblesevenarguments", "gobbleeightarguments", "gobbleninearguments", "gobbletenarguments",
         "gobbleoneoptional", "gobbletwooptionals", "gobblethreeoptionals", "gobblefouroptionals", "gobblefiveoptionals",
         --
-        "dorecurse", "doloop", "exitloop", "dostepwiserecurse", "recurselevel", "recursedepth", "dofastloopcs", "dowith",
+        "dorecurse", "doloop", "exitloop", "dostepwiserecurse", "recurselevel", "recursedepth", "dofastloopcs", "fastloopindex", "fastloopfinal", "dowith",
         --
         "newconstant", "setnewconstant", "setconstant", "setconstantvalue",
         "newconditional", "settrue", "setfalse", "settruevalue", "setfalsevalue",
@@ -323,7 +364,7 @@ return {
         "dosinglegroupempty", "dodoublegroupempty", "dotriplegroupempty", "doquadruplegroupempty", "doquintuplegroupempty",
         "permitspacesbetweengroups", "dontpermitspacesbetweengroups",
         --
-        "nopdfcompression", "maximumpdfcompression", "normalpdfcompression",
+        "nopdfcompression", "maximumpdfcompression", "normalpdfcompression", "onlypdfobjectcompression", "nopdfobjectcompression",
         --
         "modulonumber", "dividenumber",
         --
@@ -337,9 +378,11 @@ return {
         --
         "offinterlineskip", "oninterlineskip", "nointerlineskip",
         --
-        "strut", "halfstrut", "quarterstrut", "depthstrut", "setstrut", "strutbox", "strutht", "strutdp", "strutwd", "struthtdp", "begstrut", "endstrut", "lineheight",
+        "strut", "halfstrut", "quarterstrut", "depthstrut", "halflinestrut", "noheightstrut", "setstrut", "strutbox", "strutht", "strutdp", "strutwd", "struthtdp", "strutgap", "begstrut", "endstrut", "lineheight",
         "leftboundary", "rightboundary", "signalcharacter",
         --
+        "shiftbox", "vpackbox", "hpackbox", "vpackedbox", "hpackedbox",
+        --
         "ordordspacing", "ordopspacing", "ordbinspacing", "ordrelspacing",
         "ordopenspacing", "ordclosespacing", "ordpunctspacing", "ordinnerspacing",
         --
@@ -368,7 +411,8 @@ return {
         --
         "startimath", "stopimath", "normalstartimath", "normalstopimath",
         "startdmath", "stopdmath", "normalstartdmath", "normalstopdmath",
-        "normalsuperscript", "normalsubscript",
+        "normalsuperscript", "normalsubscript", "normalnosuperscript", "normalnosubscript",
+        "superscript", "subscript", "nosuperscript", "nosubscript",
         --
         "uncramped", "cramped", "triggermathstyle", "mathstylefont", "mathsmallstylefont", "mathstyleface", "mathsmallstyleface", "mathstylecommand", "mathpalette",
         "mathstylehbox", "mathstylevbox", "mathstylevcenter", "mathstylevcenteredhbox", "mathstylevcenteredvbox",
@@ -395,10 +439,12 @@ return {
         "obeylualines", "obeyluatokens",
         "startluacode", "stopluacode", "startlua", "stoplua",
         "startctxfunction","stopctxfunction","ctxfunction",
-        "startctxfunctiondefinition","stopctxfunctiondefinition", "installctxfunction",
-        "cldprocessfile", "cldloadfile", "cldcontext", "cldcommand",
+        "startctxfunctiondefinition","stopctxfunctiondefinition",
+        "installctxfunction", "installprotectedctxfunction",  "installprotectedctxscanner", "installctxscanner", "resetctxscanner",
+        "cldprocessfile", "cldloadfile", "cldloadviafile", "cldcontext", "cldcommand",
         --
         "carryoverpar",
+        "lastlinewidth",
         --
         "assumelongusagecs",
         --
@@ -408,7 +454,9 @@ return {
         "rtlhbox", "ltrhbox", "rtlvbox", "ltrvbox", "rtlvtop", "ltrvtop",
         "autodirhbox", "autodirvbox", "autodirvtop",
         "leftorrighthbox", "leftorrightvbox", "leftorrightvtop",
-        "lefttoright", "righttoleft","synchronizelayoutdirection","synchronizedisplaydirection","synchronizeinlinedirection",
+        "lefttoright", "righttoleft", "checkedlefttoright", "checkedrighttoleft",
+        "synchronizelayoutdirection","synchronizedisplaydirection","synchronizeinlinedirection",
+        "dirlre", "dirrle", "dirlro", "dirrlo",
         --
         "lesshyphens", "morehyphens", "nohyphens", "dohyphens",
         --
@@ -418,8 +466,15 @@ return {
         --
         "nospace", "nospacing", "dospacing",
         --
-        "naturalhbox", "naturalvbox", "naturalhpack", "naturalvpack",
+        "naturalhbox", "naturalvbox", "naturalvtop", "naturalhpack", "naturalvpack", "naturaltpack",
+        "reversehbox", "reversevbox", "reversevtop", "reversehpack", "reversevpack", "reversetpack",
         --
         "frule",
+        --
+        "compoundhyphenpenalty",
+        --
+        "start", "stop",
+        --
+        "unsupportedcs",
     }
 }
