@@ -7,7 +7,7 @@
 -----------------------------------------------------------------------
 
 if not modules then modules = { } end modules ['xindex-cfg'] = {
-      version = 0.18,
+      version = 0.23,
       comment = "configuration to xindex.lua",
        author = "Herbert Voss",
     copyright = "Herbert Voss",
@@ -40,7 +40,6 @@ idxnewletter      = "\\textbf"  -- Only valid if -n is not set
     Several characters in the same slot have the same 'priority'.
 ]]
 
-
 alphabet_lower = { --   for sorting
     { ' ' },  -- only for internal tests
     { 'a', 'á', 'à', 'å', 'æ', },
@@ -75,29 +74,29 @@ alphabet_lower = { --   for sorting
 }
 alphabet_upper = { -- for sorting
     { ' ' },
-    { 'A', 'Á', 'À', 'Å', 'Æ'},
+    { 'A', 'Á', 'À', 'Å', 'Æ', 'Â'},
     { 'AE', 'Ä'},
     { 'B' },
     { 'C', 'Ç' },
     { 'D' },
-    { 'E', 'È', 'É', 'Ë' },
+    { 'E', 'È', 'É', 'Ë', 'Ê' },
     { 'F' },
     { 'G' },
     { 'H' },
-    { 'I', 'Í', 'Ì', 'Ï' },
+    { 'I', 'Í', 'Ì', 'Ï', 'Î' },
     { 'J' },
     { 'K' },
     { 'L' },
     { 'M' },
     { 'N', 'Ñ' },
-    { 'O', 'Ó', 'Ò', 'Ø','Œ', 'Ø' },
+    { 'O', 'Ó', 'Ò', 'Ø','Œ', 'Ø', 'Ô' },
     { 'OE', 'Ö' },
     { 'P' },
     { 'Q' },
     { 'R' },
     { 'S', 'Š' },
     { 'T' },
-    { 'U', 'Ú', 'Ù' },
+    { 'U', 'Ú', 'Ù', 'Û' },
     { 'UE', 'Ü' },
     { 'V' },
     { 'W' },
