@@ -9,7 +9,7 @@
 -- configuration for index files of the Heidelberger Akademie der Wissenschaften
 
 if not modules then modules = { } end modules ['xindex-HAdW-eKO'] = {
-      version = 0.18,
+      version = 0.23,
       comment = "configuration to xindex.lua",
        author = "Herbert Voss",
     copyright = "Herbert Voss",
@@ -62,7 +62,7 @@ alphabet_lower = { --   for sorting
     { 'p' },
     { 'q' },
     { 'r' },
-    { 's', 'ß' },
+    { 's' },
     { 't' },
     { 'u', 'ú', 'ù', 'ü' },
     { 'v' },
