@@ -6,8 +6,8 @@
 
 local ProvidesLuaModule = { 
     name          = "luaotfload-auxiliary",
-    version       = "3.00",       --TAGVERSION
-    date          = "2019-09-13", --TAGDATE
+    version       = "3.14",       --TAGVERSION
+    date          = "2020-05-06", --TAGDATE
     description   = "luaotfload submodule / auxiliary functions",
     license       = "GPL v2.0"
 }
@@ -33,12 +33,20 @@ local getmetatable          = getmetatable
 local setmetatable          = setmetatable
 local utfcodepoint          = utf8.codepoint
 local stringlower           = string.lower
+local stringupper           = string.upper
 local stringformat          = string.format
 local stringgsub            = string.gsub
 local stringbyte            = string.byte
 local stringfind            = string.find
 local tablecopy             = table.copy
 
+local harf = luaotfload.harfbuzz
+local GSUBtag, GPOStag
+if harf then
+  GSUBtag = harf.Tag.new("GSUB")
+  GPOStag = harf.Tag.new("GPOS")
+end
+
 local aux                   = { }
 local luaotfload_callbacks  = { }
 
@@ -101,6 +109,14 @@ luaotfload_callbacks [#luaotfload_callbacks + 1] = {
   "patch_font", set_sscale_dimens, "set_sscale_dimens",
 }
 
+-- Starting with LuaTeX 1.11.2, this is a more reliable way of selecting the
+-- right font than relying on psnames
+luaotfload_callbacks [#luaotfload_callbacks + 1] = {
+  "patch_font", function(fontdata)
+    fontdata.subfont = fontdata.specification.sub or 1
+  end, "set_font_index",
+}
+
 local default_units = 1000
 
 --- fontobj -> int
@@ -199,7 +215,7 @@ local function determine_capheight(fontdata)
     local refchar = characters [refcp]
     if refchar then
       logreport ("both", 4, "aux",
-                 "picked height of character ‘%s’ (U+%d) as \\fontdimen8 \z
+                 "picked height of character '%s' (U+%04X) as \\fontdimen8 \z
                   candidate",
                  capheight_reference_chars [i], refcp)
       return refchar.height
@@ -349,15 +365,27 @@ function aux.slot_of_name(font_id, glyphname, unsafe)
 
   local tfmdata = identifiers [font_id]
   if not tfmdata then return raw_slot_of_name (font_id, glyphname) end
-  local resources = tfmdata.resources  if not resources then return false end
-  local unicodes  = resources.unicodes if not unicodes  then return false end
+  local hbdata = tfmdata.hb
+  if hbdata then
+    local hbshared = hbdata.shared
+    local nominals = hbshared.nominals
+    local hbfont = hbshared.font
 
-  local unicode = unicodes [glyphname]
-  if unicode then
-    if type (unicode) == "number" then
-      return unicode
-    else
-      return unicode [1] --- for multiple components
+    local gid = hbfont:get_glyph_from_name(glyphname)
+    if gid ~= nil then
+      return nominals[gid] or gid + hbshared.gid_offset
+    end
+  else
+    local resources = tfmdata.resources  if not resources then return false end
+    local unicodes  = resources.unicodes if not unicodes  then return false end
+
+    local unicode = unicodes [glyphname]
+    if unicode then
+      if type (unicode) == "number" then
+        return unicode
+      else
+        return unicode [1] --- for multiple components
+      end
     end
   end
   return false
@@ -401,6 +429,19 @@ function aux.name_of_slot(codepoint)
   return false
 end
 
+--[[doc--
+
+  Get the GID of the glyph associated with a given name.
+
+--doc]]--
+function aux.gid_of_name(font_id, glyphname)
+  local slot = aux.slot_of_name(font_id, glyphname)
+  if not slot then return end
+  local tfmdata = identifiers[font_id] or font.fonts[font_id]
+  -- assert(tfmdata) -- Otherwise slot_of_name would have failed already
+  return tfmdata.characters[slot].index or slot
+end
+
 -----------------------------------------------------------------------
 ---                 features / scripts / languages
 -----------------------------------------------------------------------
@@ -412,6 +453,11 @@ local function get_features(tfmdata)
   return features
 end
 
+local function get_hbface(tfmdata)
+  if not tfmdata.hb then return end
+  return tfmdata.hb.shared.face
+end
+
 --[[doc--
 This function, modeled after “check_script()” from fontspec, returns
 true if in the given font, the script “asked_script” is accounted for in at
@@ -429,9 +475,22 @@ function aux.provides_script(font_id, asked_script)
     return false
   end
   asked_script = stringlower(asked_script)
-  if font_id and font_id > 0 then
-    local tfmdata = identifiers[font_id]
-    if not tfmdata then return false end
+  local tfmdata = identifiers[font_id]
+  if not tfmdata then
+    logreport ("log", 0, "aux", "no font with id %d", font_id)
+    return false
+  end
+  local hbface = get_hbface(tfmdata)
+  if hbface then
+    local script = harf.Tag.new(asked_script == "dflt" and "DFLT"
+                                                        or asked_script)
+    for _, tag in next, { GSUBtag, GPOStag } do
+      if hbface:ot_layout_find_script(tag, script) then
+        return true
+      end
+    end
+    return false
+  else
     local features = get_features (tfmdata)
     if features == false then
       logreport ("log", 1, "aux", "font no %d lacks a features table", font_id)
@@ -452,7 +511,6 @@ function aux.provides_script(font_id, asked_script)
                "font no %d (%s) defines no feature for script %s",
                font_id, fontname, asked_script)
   end
-  logreport ("log", 0, "aux", "no font with id %d", font_id)
   return false
 end
 
@@ -477,8 +535,30 @@ function aux.provides_language(font_id, asked_script, asked_language)
     return false
   end
   asked_script   = stringlower(asked_script)
-  asked_language = stringlower(asked_language)
-  if font_id and font_id > 0 then
+  local tfmdata = identifiers[font_id]
+  if not tfmdata then
+    logreport ("log", 0, "aux", "no font with id %d", font_id)
+    return false
+  end
+  local hbface = get_hbface(tfmdata)
+  if hbface then
+    asked_language  = stringupper(asked_language)
+    if asked_language == "DFLT" then
+      return aux.provides_script(font_id, asked_script)
+    end
+    local script = harf.Tag.new(asked_script == "dflt" and "DFLT"
+                                                        or asked_script)
+    local language = harf.Tag.new(asked_language == "DFLT" and "dflt"
+                                                            or asked_language)
+    for _, tag in next, { GSUBtag, GPOStag } do
+      local _, script_idx = hbface:ot_layout_find_script(tag, script)
+      if hbface:ot_layout_find_language(tag, script_idx, language) then
+        return true
+      end
+    end
+    return false
+  else
+    asked_language  = stringlower(asked_language)
     local tfmdata = identifiers[font_id]
     if not tfmdata then return false end
     local features = get_features (tfmdata)
@@ -504,9 +584,8 @@ function aux.provides_language(font_id, asked_script, asked_language)
                "font no %d (%s) defines no feature "
                .. "for script %s with language %s",
                font_id, fontname, asked_script, asked_language)
+    return false
   end
-  logreport ("log", 0, "aux", "no font with id %d", font_id)
-  return false
 end
 
 --[[doc--
@@ -558,12 +637,32 @@ function aux.provides_feature(font_id,        asked_script,
     return false
   end
   asked_script    = stringlower(asked_script)
-  asked_language  = stringlower(asked_language)
   asked_feature   = lpegmatch(strip_garbage, asked_feature)
 
-  if font_id > 0 then
-    local tfmdata  = identifiers[font_id]
-    if not tfmdata then return false end
+  local tfmdata = identifiers[font_id]
+  if not tfmdata then
+    logreport ("log", 0, "aux", "no font with id %d", font_id)
+    return false
+  end
+  local hbface = get_hbface(tfmdata)
+  if hbface then
+    asked_language  = stringupper(asked_language)
+    local script = harf.Tag.new(asked_script == "dflt" and "DFLT"
+                                                        or asked_script)
+    local language = harf.Tag.new(asked_language == "DFLT" and "dflt"
+                                                            or asked_language)
+    local feature = harf.Tag.new(asked_feature)
+
+    for _, tag in next, { GSUBtag, GPOStag } do
+      local _, script_idx = hbface:ot_layout_find_script(tag, script)
+      local _, language_idx = hbface:ot_layout_find_language(tag, script_idx, language)
+      if hbface:ot_layout_find_feature(tag, script_idx, language_idx, feature) then
+        return true
+      end
+    end
+    return false
+  else
+    asked_language  = stringlower(asked_language)
     local features = get_features (tfmdata)
     if features == false then
       logreport ("log", 1, "aux", "font no %d lacks a features table", font_id)
@@ -588,8 +687,6 @@ function aux.provides_feature(font_id,        asked_script,
                "font no %d (%s) does not define feature %s for script %s with language %s",
                font_id, fontname, asked_feature, asked_script, asked_language)
   end
-  logreport ("log", 0, "aux", "no font with id %d", font_id)
-  return false
 end
 
 -----------------------------------------------------------------------
@@ -632,7 +729,7 @@ end
 --- https://github.com/lualatex/luaotfload/pull/61#issuecomment-17776975
 
 --- string -> (int * int)
-local scan_external_dir = function (dir)
+local function scan_external_dir (dir)
   local old_names, new_names = names.data()
   if not old_names then
     old_names = load_names()
@@ -821,7 +918,7 @@ end
 local function inject_callbacks (lst)
   if not lst and next (lst) then return false end
 
-  local inject = function (def)
+  local function inject (def)
     local cb, fn, id = unpack (def)
     cb = tostring (cb)
     id = tostring (id)
